@@ -12,11 +12,9 @@ use std::fmt::Debug;
 use std::str;
 pub const TWO_TIMES_SECRET_KEY_SIZE: usize = 64;
 use super::traits::{ECPoint, ECScalar};
-use crate::arithmetic::traits::Converter;
 use crate::cryptographic_primitives::hashing::hash_sha256::HSha256;
 use crate::cryptographic_primitives::hashing::traits::Hash;
-use serde::de;
-use serde::de::{MapAccess, SeqAccess, Visitor};
+use serde::de::{self, Error, MapAccess, SeqAccess, Visitor};
 use serde::ser::SerializeStruct;
 use serde::ser::{Serialize, Serializer};
 use serde::{Deserialize, Deserializer};
@@ -24,7 +22,7 @@ use std::fmt;
 use std::ops::{Add, Mul};
 pub type SK = Fe;
 pub type PK = GeP3;
-use crate::arithmetic::traits::{Modulo, Samplable};
+use crate::arithmetic::traits::*;
 use crate::BigInt;
 use crate::ErrorKey::{self, InvalidPublicKey};
 #[cfg(feature = "merkle")]
@@ -58,7 +56,9 @@ impl Zeroize for Ed25519Scalar {
         atomic::compiler_fence(atomic::Ordering::SeqCst);
     }
 }
-impl ECScalar<SK> for Ed25519Scalar {
+impl ECScalar for Ed25519Scalar {
+    type SecretKey = SK;
+
     // we chose to multiply by 8 (co-factor) all group elements to work in the prime order sub group.
     // each random fe is having its 3 first bits zeroed
     fn new_random() -> Ed25519Scalar {
@@ -83,7 +83,7 @@ impl ECScalar<SK> for Ed25519Scalar {
     }
 
     fn from(n: &BigInt) -> Ed25519Scalar {
-        let mut v = BigInt::to_vec(&n);
+        let mut v = BigInt::to_bytes(&n);
         if v.len() > TWO_TIMES_SECRET_KEY_SIZE {
             v = v[0..TWO_TIMES_SECRET_KEY_SIZE].to_vec();
         }
@@ -103,7 +103,7 @@ impl ECScalar<SK> for Ed25519Scalar {
         let t1 = &self.fe.to_bytes()[0..self.fe.to_bytes().len()];
         let mut t2 = t1.to_vec();
         t2.reverse();
-        BigInt::from(&t2[0..self.fe.to_bytes().len()])
+        BigInt::from_bytes(&t2[0..self.fe.to_bytes().len()])
     }
 
     fn q() -> BigInt {
@@ -159,7 +159,7 @@ impl ECScalar<SK> for Ed25519Scalar {
 
     fn invert(&self) -> Ed25519Scalar {
         let self_bn = self.to_big_int();
-        let inv = self_bn.invert(&FE::q()).unwrap();
+        let inv = BigInt::mod_inv(&self_bn, &FE::q()).unwrap();
         let inv_fe: FE = ECScalar::from(&inv);
         inv_fe
     }
@@ -238,7 +238,7 @@ impl<'de> Visitor<'de> for Ed25519ScalarVisitor {
     }
 
     fn visit_str<E: de::Error>(self, s: &str) -> Result<Ed25519Scalar, E> {
-        let v = BigInt::from_str_radix(s, 16).expect("Failed in serde");
+        let v = BigInt::from_hex(s).map_err(E::custom)?;
         Ok(ECScalar::from(&v))
     }
 }
@@ -268,20 +268,23 @@ impl Zeroize for Ed25519Point {
     }
 }
 
-impl Ed25519Point {
-    pub fn base_point2() -> Ed25519Point {
+impl ECPoint for Ed25519Point {
+    type SecretKey = SK;
+    type PublicKey = PK;
+    type Scalar = Ed25519Scalar;
+
+    fn base_point2() -> Ed25519Point {
         let g: GE = ECPoint::generator();
         let hash = HSha256::create_hash(&[&g.bytes_compressed_to_big_int()]);
         let hash = HSha256::create_hash(&[&hash]);
-        let bytes = BigInt::to_vec(&hash);
+        let bytes = BigInt::to_bytes(&hash);
         let h: GE = ECPoint::from_bytes(&bytes[..]).unwrap();
         Ed25519Point {
             purpose: "random",
             ge: h.get_element(),
         }
     }
-}
-impl ECPoint<PK, SK> for Ed25519Point {
+
     fn generator() -> Ed25519Point {
         let vec_1: [u8; 32];
         vec_1 = [
@@ -313,7 +316,7 @@ impl ECPoint<PK, SK> for Ed25519Point {
     }
 
     fn bytes_compressed_to_big_int(&self) -> BigInt {
-        BigInt::from(self.ge.to_bytes()[0..self.ge.to_bytes().len()].as_ref())
+        BigInt::from_bytes(self.ge.to_bytes()[0..self.ge.to_bytes().len()].as_ref())
     }
 
     // from_bytes will return Ok only if the bytes encode a valid point.
@@ -486,7 +489,7 @@ impl Serialize for Ed25519Point {
         S: Serializer,
     {
         let bytes = self.pk_to_key_slice();
-        let bytes_as_bn = BigInt::from(&bytes[..]);
+        let bytes_as_bn = BigInt::from_bytes(&bytes[..]);
         let padded_bytes_hex = format!("{:0>64}", bytes_as_bn.to_hex());
         let mut state = serializer.serialize_struct("ed25519CurvePoint", 1)?;
         state.serialize_field("bytes_str", &padded_bytes_hex)?;
@@ -517,12 +520,13 @@ impl<'de> Visitor<'de> for Ed25519PointVisitor {
     where
         V: SeqAccess<'de>,
     {
-        let bytes_str = dbg!(seq
+        let bytes_str = seq
             .next_element()?
-            .ok_or_else(|| panic!("deserialization failed"))?);
-        let bytes_bn = BigInt::from_hex(bytes_str);
-        let bytes = BigInt::to_vec(&bytes_bn);
-        Ok(Ed25519Point::from_bytes(&bytes[..]).expect("error deserializing point"))
+            .ok_or_else(|| V::Error::invalid_length(0, &"a single element"))?;
+        let bytes_bn = BigInt::from_hex(bytes_str).map_err(V::Error::custom)?;
+        let bytes = BigInt::to_bytes(&bytes_bn);
+        Ed25519Point::from_bytes(&bytes[..])
+            .map_err(|_| V::Error::custom("failed to parse ed25519 point"))
     }
 
     fn visit_map<E: MapAccess<'de>>(self, mut map: E) -> Result<Ed25519Point, E::Error> {
@@ -534,17 +538,18 @@ impl<'de> Visitor<'de> for Ed25519PointVisitor {
                 "bytes_str" => {
                     bytes_str = String::from(v);
                 }
-                _ => panic!("deserialization failed!"),
+                _ => return Err(E::Error::unknown_field(key, &["bytes_str"])),
             }
         }
 
-        let bytes_bn = BigInt::from_hex(&bytes_str);
-        let bytes = BigInt::to_vec(&bytes_bn);
+        let bytes_bn = BigInt::from_hex(&bytes_str).map_err(E::Error::custom)?;
+        let bytes = BigInt::to_bytes(&bytes_bn);
 
-        Ok(Ed25519Point::from_bytes(&bytes[..]).expect("error deserializing point"))
+        Ed25519Point::from_bytes(&bytes[..]).map_err(|_| E::Error::custom("invalid ed25519 point"))
     }
 }
 
+#[allow(clippy::many_single_char_names)]
 //helper function, based on https://ed25519.cr.yp.to/python/ed25519.py
 pub fn xrecover(y_coor: BigInt) -> BigInt {
     //   let d = "37095705934669439343138083508754565189542113879843219016388785533085940283555";
@@ -555,9 +560,9 @@ pub fn xrecover(y_coor: BigInt) -> BigInt {
     let d_d = expmod(&BigInt::from(121_666), &(q.clone() - BigInt::from(2)), &q);
 
     let d_bn = d_n * d_d;
-    let y_sqr = y_coor.clone() * y_coor.clone();
+    let y_sqr = y_coor.clone() * y_coor;
     let u = y_sqr.clone() - one.clone();
-    let v = y_sqr * d_bn.clone() + one.clone();
+    let v = y_sqr * d_bn + one;
     let v_inv = expmod(&v, &(q.clone() - BigInt::from(2)), &q);
 
     let x_sqr = u * v_inv;
@@ -569,8 +574,8 @@ pub fn xrecover(y_coor: BigInt) -> BigInt {
         let i = expmod(&BigInt::from(2i32), &q_minus_1_div_4, &q);
         x = BigInt::mod_mul(&x, &i, &q);
     }
-    if x.modulus(&BigInt::from(2i32)) != BigInt::zero() {
-        x = q.clone() - x.clone();
+    if BigInt::modulus(&x, &BigInt::from(2i32)) != BigInt::zero() {
+        x = q - x.clone();
     }
 
     x
@@ -580,29 +585,30 @@ pub fn xrecover(y_coor: BigInt) -> BigInt {
 pub fn expmod(b: &BigInt, e: &BigInt, m: &BigInt) -> BigInt {
     let one = BigInt::one();
     if e.clone() == BigInt::zero() {
-        return one.clone();
+        return one;
     };
     let t_temp = expmod(b, &(e.clone() / BigInt::from(2u32)), m);
     let mut t = BigInt::mod_pow(&t_temp, &BigInt::from(2u32), m);
 
-    if e.clone().modulus(&BigInt::from(2)) != BigInt::zero() {
+    if BigInt::modulus(&e, &BigInt::from(2)) != BigInt::zero() {
         t = BigInt::mod_mul(&t, b, m);
     }
     t
 }
 
-#[cfg(feature = "ec_ed25519")]
 #[cfg(test)]
 mod tests {
-    use super::Ed25519Point;
-    use crate::arithmetic::traits::{Converter, Modulo};
+    use super::{Ed25519Point, Ed25519Scalar};
+    use crate::arithmetic::traits::*;
     use crate::elliptic::curves::traits::ECPoint;
     use crate::elliptic::curves::traits::ECScalar;
     use crate::BigInt;
-    use crate::{FE, GE};
-    use serde_json;
+
+    type GE = Ed25519Point;
+    type FE = Ed25519Scalar;
 
     #[test]
+    #[allow(clippy::op_ref)] // Enables type inference.
     fn test_serdes_pk() {
         let mut pk = GE::generator();
         let mut s = serde_json::to_string(&pk).expect("Failed in serialization");
@@ -639,6 +645,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(clippy::op_ref)] // Enables type inference.
     fn bincode_pk() {
         let pk = GE::generator();
         let encoded = bincode::serialize(&pk).unwrap();
@@ -649,6 +656,7 @@ mod tests {
 
     #[test]
     #[should_panic]
+    #[allow(clippy::op_ref)] // Enables type inference.
     fn test_serdes_bad_pk() {
         let pk = GE::generator();
         let s = serde_json::to_string(&pk).expect("Failed in serialization");
@@ -673,9 +681,9 @@ mod tests {
         let b: FE = ECScalar::new_random();
         let a_minus_b_fe: FE = a.sub(&b.get_element());
         let base: GE = ECPoint::generator();
-        let point_ab1 = &base * &a_minus_b_fe;
-        let point_a = &base * &a;
-        let point_b = &base * &b;
+        let point_ab1 = base * a_minus_b_fe;
+        let point_a = base * a;
+        let point_b = base * b;
         let point_ab2 = point_a.sub_point(&point_b.get_element());
         assert_eq!(point_ab1, point_ab2);
     }
@@ -684,11 +692,11 @@ mod tests {
     fn test_add_point() {
         let a: FE = ECScalar::new_random();
         let b: FE = ECScalar::new_random();
-        let a_plus_b_fe = a.clone() + &b;
+        let a_plus_b_fe = a + b;
         let base: GE = ECPoint::generator();
-        let point_ab1 = &base * &a_plus_b_fe;
-        let point_a = &base * &a;
-        let point_b = &base * &b;
+        let point_ab1 = base * a_plus_b_fe;
+        let point_a = base * a;
+        let point_b = base * b;
         let point_ab2 = point_a.add_point(&point_b.get_element());
 
         assert_eq!(point_ab1, point_ab2);
@@ -698,7 +706,7 @@ mod tests {
     fn test_add_scalar() {
         let a: FE = ECScalar::new_random();
         let zero: FE = FE::zero();
-        let a_plus_zero: FE = a.clone() + zero;
+        let a_plus_zero: FE = a + zero;
 
         assert_eq!(a_plus_zero, a);
     }
@@ -709,7 +717,7 @@ mod tests {
             10, 10, 10, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
             0, 10, 10, 10,
         ];
-        let a_bn = BigInt::from(&a[..]);
+        let a_bn = BigInt::from_bytes(&a[..]);
         let a_fe: FE = ECScalar::from(&a_bn);
 
         let five = BigInt::from(5);
@@ -725,10 +733,10 @@ mod tests {
     fn test_mul_point() {
         let a: FE = ECScalar::new_random();
         let b: FE = ECScalar::new_random();
-        let a_mul_b_fe = a.clone() * &b;
+        let a_mul_b_fe = a * b;
         let base: GE = ECPoint::generator();
-        let point_ab1 = &base * &a_mul_b_fe;
-        let point_a = &base * &a;
+        let point_ab1 = base * a_mul_b_fe;
+        let point_a = base * a;
         let point_ab2 = point_a.scalar_mul(&b.get_element());
 
         assert_eq!(point_ab1, point_ab2);
@@ -741,7 +749,7 @@ mod tests {
         let a_bn = a.to_big_int();
 
         let a_inv = a.invert();
-        let a_inv_bn_1 = a_bn.invert(&FE::q()).unwrap();
+        let a_inv_bn_1 = BigInt::mod_inv(&a_bn, &FE::q()).unwrap();
         let a_inv_bn_2 = a_inv.to_big_int();
 
         assert_eq!(a_inv_bn_1, a_inv_bn_2);
@@ -770,7 +778,7 @@ mod tests {
         let g: GE = ECPoint::generator();
 
         let fe: FE = ECScalar::from(&BigInt::from(1));
-        let b_tag = &g * &fe;
+        let b_tag = g * fe;
         assert_eq!(b_tag, g);
     }
 
@@ -782,9 +790,9 @@ mod tests {
             0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
             0, 0, 1,
         ];
-        let tv_bn = BigInt::from(&test_vec[..]);
+        let tv_bn = BigInt::from_bytes(&test_vec[..]);
         let test_fe: FE = ECScalar::from(&tv_bn);
-        let test_ge = g * &test_fe;
+        let test_ge = g * test_fe;
         let test_ge_bytes = test_ge.get_element().to_bytes();
         let test_ge2: GE = ECPoint::from_bytes(&test_ge_bytes[..]).unwrap();
         let eight: FE = ECScalar::from(&BigInt::from(8));
@@ -803,8 +811,8 @@ mod tests {
     fn test_xy_coor() {
         let g: GE = GE::generator();
         assert_eq!(
-            g.x_coor().unwrap().to_str_radix(10),
-            "15112221349535400772501151409588531511454012693041857206046113283949847762202"
+            g.x_coor().unwrap().to_hex(),
+            "216936d3cd6e53fec0a4e231fdd6dc5c692cc7609525a7b2c9562d608f25d51a"
         );
     }
 }
